# Practical introduction to the representation theory of space groups

The following note is a practical description of calculating character tables for symmorphic space groups and their use in constructing wave functions that are invariant under the same symmetries, with particular focus on some architectures used in NetKet. An accessible reference for more mathematical detail is [^bradley].

[^bradley]: [The Mathematical Theory of Symmetry in Solids.](https://global.oup.com/academic/product/the-mathematical-theory-of-symmetry-in-solids-9780199582587?cc=gb&lang=en&) C. J. Bradley and A. P. Cracknell (1972). Clarendon Press, Oxford. (Chapter 3 is an accessible but comprehensive summary of the construction for both symmorphic and non-symmorphic groups; Chapter 4 covers every loose points of pure maths.)

## Space groups and point groups

A **space group** is the set of all symmetries of a system with discrete translational invariance. It consists of the same translations as well as symmetries that change its orientation (rotations, reflections, glides, screws, etc.). There is a finite number of possible space groups in any number of dimensions, which (and all the other objects we mention here) have been tabulated by crystallographers. In 2D, there are 17 space group (also known as *wallpaper groups*); in 3D, there are 230. We use $S$ to denote it.

The **translation group** of the same system is the subgroup of the space group that consists of all translations by lattice vectors. It is a normal subgroup: it is intuitively clear that if $t$ is a lattice translation, and $g$ is any symmetry in the space group, then $g^{-1}tg$ is another pure lattice translation, since applying $g$ undoes any change in orientation that $g$ might have caused.

Orientation-changing symmetries are described in terms of the **point group** (usually denoted with $P$ here), which can be defined in two more-or-less equivalent ways:

1. As a normal subgroup, the left and right cosets of the translation group are the same, and can be thought of as classes of symmetries that are equivalent up to translation. These classes form a quotient group, which is the point group.
1. We can alternatively choose a representative symmetry from each of these classes such that they form a subgroup of the space group themselves. For example, the point group of the square lattice is the dihedral group $D_8$, which can be represented by rotations around, and reflections across, any given lattice point. Now, the space group is the group generated by both the translations and these representative point group symmetries.

The first definition can mechanically be applied to any space group; the second tacitly assumes that one can choose an origin that is invariant under all representative point group symmetries.
If such is the case, the space group is called **symmorphic** and it is the semidirect product of the translation group and the point group in the second definition.
However, non-symmorphic space groups also exist: their generators must include glide or screw symmetries that leave no point invariant.  
For simplicity, the rest of these notes focuses on symmorphic space groups. Non-symmorphic groups come with a bunch of technical difficulties, which are not important for the core message here; [^bradley] deals with them in detail.

## Representation theory in quantum mechanics

Group theory, to the boon and bane of physicists, is an indispensable tool to quantum mechanics. In our context, it enters via Wigner's theorem:

> If $\Psi$ is an eigenstate of the Hamiltonian $H$ with some energy, and $g$ is a unitary symmetry that commutes with $H$, $g\Psi$ is an eigenstate of $H$ with the same energy.

This means that the matrices that describe the action of the space group of $H$ in the eigenbasis of the same are block diagonal, only connecting degenerate states to one another. Barring accidental degeneracies, each of the diagonal blocks consists of an **irreducible representation (irrep)** of the space group. Therefore, all information about how eigenstates transform under spatial symmetries, and about symmetry-protected degeneracies, is encoded in the classification of same irreps.

This classification is normally done in terms of **character tables** that list the traces of the matrices that correspond to each group element.

* The trace is invariant under unitary transformations $U_g \to UU_gU^{-1}$, therefore, the character is independent of the basis used to write the representation in.
* Since group representations always consist of unitary matrices, $\chi(hgh^{-1}) = \mathrm{tr}(U_h U_g U_h^{-1}) = \mathrm{tr} U_g = \chi(g)$. That is, the character of a group element only depends on its conjugacy class.
* Characters of inequivalent (up to unitary transformations) irreps are orthogonal:
$$ \frac1{|G|} \sum_{g\in G} \chi_i(g) \overline{\chi_j(g)} = \delta_{ij}. $$
This can be used to decompose an arbitrary representation into its irrep content.
* It follows from the previous two that the number of inequivalent irreps equals the number of conjugacy classes.

## Representation theory of space groups

We now want to construct the character table (or rather, characters of irreps corresponding to specific physical properties) of a symmorphic space group. We do this in two steps:

1. We construct the irreps of the translation group. This is trivial, physically intuitive, and since we use cosets of it to define the point group, it is a mathematically useful starting point, too.
2. We consider the action of the point group on wave functions that transform according to given irreps of the translation group:
    1. first, we focus on point group operations that leave the translational representation unchanged;
    2. finally, we consider the actions of all other elements of the point group.

### Representations of the translation group

The translation group of an infinite lattice in $d$ dimensions is always isomorphic to $\mathbb{Z}^d$, where each $\mathbb{Z}$ factor is generated by translation with one of the vectors $\vec a_i$ that span the lattice.

The irreps of $\mathbb{Z}$ are all one-dimensional (for it is abelian): the character (only entry in the representation) of the generator $t$ can be any unit complex number $z=e^{-ik}$ [^minus_k], whence the character of $t^n$ is $z^n = e^{-ikn}$. Distinct values of $k$ lie between $-\pi$ and $\pi$: it is the (one-dimensional) quasimomentum.

[^minus_k]: The choice of setting $z=e^{-ik}$ rather than $e^{+ik}$ is a matter of convention. We choose to follow the convention in [^bradley]; this is consistent with Bloch functions of the form $\psi_{\vec k}(\vec r) = e^{+i\vec k\cdot\vec r} u(\vec r)$.

All characters of direct products of groups can be generated as a "direct product" of characters of the factors. In our case, characters of $\mathbb{Z}^d$ are still one-dimensional and are labelled by $k_1,k_2,\dots, k_d$: the character of translation along $n_1\vec a_1 + \dots + n_d \vec a_d$ is $e^{-i(k_1n_1+\dots+k_dn_d)}$. Physically, the $k_i$ are components of the wave vector (restricted to a Brillouin zone) in terms of the reciprocal lattice vectors.

Here, we are more interested in finite simulation boxes with periodic boundary conditions, rather than the thermodynamic limit. Irreps of the now finite translation group can still be written in terms of wave vectors, so long as they satisfy $e^{-i\vec k\vec R} = 1$ for all "lattice vectors" defined by the periodic boundary conditions. This restriction doesn't change anything in the following discussion, as long as this "lattice" is invariant under all point-group symmetries of the original lattice. In practice, this can be achieved by using simulation boxes measuring the same number of (primitive or conventional) unit cells along each dimension.

### Little groups and stars

Before going on to constructing irreps of the full space group, we consider the effect of point group operations on wave vectors (i.e., wave functions that transform according to the corresponding irrep of the translation group).
Each operation acts in the geometrically obvious way on wave vectors (though that has to be converted into the reciprocal basis, which requires some care).

* The orbit of a wave vector $\vec k$ (i.e. the set of vectors $g\vec k$ for elements $g$ of the point group) is called the **star** of $\vec k$, denoted by $\vec k\star$. Individual wave vectors in a star are called its *arms.* Any irrep of the space group that contains wave functions with wave vector $\vec k$ necessarily contains all other arms of its star, too.
* The stabiliser of wave vector $\vec k$ *in the point group* (i.e. the set of point group operations $g$ such that $g\vec k = \vec k$) is called the **little co-group** at $\vec k$, denoted by $P(\vec k)$. As any stabiliser, it is a subgroup of $P$.
* The **little group** is the semidirect product of the translation group and the little co-group in the second definition of the point group, or the union of cosets used to define the little co-group under the first definition. It is a subgroup *of the space group,* denoted by $S(\vec k)$.

### Small irreps of the little group

It is clear that any irrep of the full space group must cover complete stars, so they cannot really be labelled by a single wave vector. However, it is still instructive to first restrict ourselves to the sector of the Hilbert space with a single $\vec k$: this also means we focus on $S(\vec k)$, since other space group operations would take us out of this sector.

The representation of translation symmetries in this sector are trivial (the representation matrices are just phase factors that multiply the identity matrix), so any symmetry-protected degeneracy has to come from point-group symmetries. Therefore, irreps of the little group (known as **small irreps**) are in a one-to-one mapping with irreps of the little co-group:

* the characters of the pure point group symmetries can be read off from the character tables of the little co-group (this is a small crystallographic point group, with ready-made character tables);
* point group + translation symmetries gain a phase factor due to the translation:
$$\chi(pt_{\vec r}) = \chi(t_\vec{r}p) = \chi(p) e^{-i\vec k\cdot\vec r}.$$
  * Note that in general $pt_\vec{r} \neq t_{\vec r}p$; however, they belong to the same conjugacy class of the little group ($p^{-1}(pt)p = tp$), so we expect that their characters be equal.
  * In fact, we can say a bit more than that to reassure ourselves about the correctness of this construction.
  For many $\vec r$, $pt_\vec{r}$ is conjugate to $p$: we will find that for such $\vec r$, $e^{-i\vec k \cdot\vec r} = 1$, so $\chi(pt_\vec{r}) = \chi(p)$.  
  For example, consider the square lattice and let $p$ be a mirror across the $x$ axis. This mirror belongs to the little group for $\vec k = (k_x,0)$ and $(k_x,\pi)$.
  Now, if $\vec r = 2n\hat y$ for some integer $n$, $pt_\vec{r} = t^{-1}_{n\hat y} pt_{n\hat y}$, so it is conjugate to $p$.
  But then indeed, $\vec k\cdot\vec r = 0$ or $2n\pi$, as required.

### Combining small irreps into full irreps

Let us now consider the effect of the full point group on these small irreps.
Point-group symmetries outside $P(\vec k)$ generate states for all arms of $\vec k\star$: these are degenerate with the original one(s), so they must belong to the same irrep of the space group.
This irrep can be thought of as a direct sum of the small irreps for all arms of the star that are generated from one another by point-group symmetries.

We now explicitly construct the matrices of this irrep in a wave-vector basis.
Let $h$ be an element of the point group such that $h\vec k = \vec k'$.
Then, the little co-group at $\vec k'$ is the set $\{hgh^{-1}: g\in P(\vec k)\}$: this is another subgroup of the point group, isomorphic to $P(\vec k)$, but generally distinct from it.
Now, given an explicit orthonormal basis $\{\psi_i\}$ of the small irrep at $\vec k$, $\{h\psi_i\}$ forms a basis of the corresponding small irrep at $\vec k'$.
Furthermore, if $g\in P(\vec k)$ is represented by $U^g$ in the former, meaning that
$$ g(a_i\psi_i) = (U^g_{ij}a_j)\psi_i,$$
$hgh^{-1} \in P(\vec k')$ is represented by the same matrix in the latter:
$$ hgh^{-1} (a_i h\psi_i) = hg(a_i\psi_i) = h(U^g_{ij}a_j)\psi_i = (U^g_{ij}a_j)(h\psi_i).$$
This also implies that the characters of $g\in P(\vec k)$ and $hgh^{-1}\in P(\vec k')$ are equal in the corresponding small irreps.

Note, however, that the choice of $h$ is not unique: if $h\vec k = \vec k'$, then $h'\vec k = \vec k'$ too, where $h' = jh$ for some $j\in P(\vec k')$.
$hgh^{-1}$ and $h'g{h'}^{-1}$ may also be different elements of $P(\vec k')$, but this is OK: $h'g{h'}^{-1} = jhgh^{-1}j^{-1}$, so they belong to the same conjugacy class of $P(\vec k')$, so their characters are expected to be equal anyway.
In terms of irrep matrices, choosing a different $h$ results in a unitary basis transformation, which leaves the characters unchanged.

Finally, we construct a basis of the full space-group irrep as a direct sum of the small irrep bases at all arms of the star. Since the character of a group element is the trace of the resulting irrep matrices, only the wave-vector blocks left in place by the same contribute to it. That is, the character of $g$ is simply the sum of the corresponding small-irrep characters over wave vectors $\vec k$ such that $g\in S(\vec k)$.

It is intuitively clear that the characters generated by this method are irreps. We do not present a formal proof here, but one is available in [^bradley], together with a proof that this construction generates every irrep precisely once.

### A practical algorithm for constructing characters

To summarise the discussion above, here follows an algorithm that generates the characters of an irrep of the space group. The input specifies a wave vector $\vec k$ and a set of characters $\chi^0$ of the corresponding little co-group (it might be convenient to supply this as a function over the whole point group, setting characters outside the little co-group to 0). The explicit irrep-matrix construction above is inconvenient in that one has to construct the star of $\vec k$ without repetitions and choose an explicit representative $h$ of the corresponding coset of $P(\vec k)$. If we only want to generate characters, this issue can be sidestepped by summing the equivalent characters for all $h$ corresponding to each $\vec k'$. Doing so simply overcounts the characters by a factor of $|P(\vec k)|$, since the $h$ corresponding to a given arm form a coset of $P(\vec k)$ in $P$ and all of them return the same characters. Therefore, we do the following:

1. Set $\chi(s)=0$ for all $s\in S$.
2. For all $p$ in the point group:
    1. Let $\vec k' = p\vec k$.
    2. For all $g\in P(\vec k)$:
        1. Let $g' = pgp^{-1}$ and $\chi'(g') = \chi^0(g)$.
        2. For all lattice translations $t_\vec{r}$, add $\chi'(g') e^{-i\vec k' \vec r}$ to $\chi(g't_\vec{r})$
3. Divide all $\chi(s)$ by $|P(\vec k)|$ and return them

### Examples

#### Trivial little co-group

A generic wave vector is not conserved by any point-group symmetry, hence its little co-group is trivial. The corresponding star has $|P|$ arms; the small irreps corresponding to each are  defined for translations only, and their characters (i.e., the only entries in the representation matrices) are phase factors $e^{i\vec k\cdot\vec r}$. Therefore, the characters of the full irrep are 
$$ \chi(t_\vec{r}) = \sum_{\vec k' \in \vec k\star} e^{-i\vec k\cdot\vec r}; \chi(s)=0\ \textrm{otherwise}. $$ 

#### Little group = space group

The opposite extreme is a wave vector (e.g., $\vec k = 0$) conserved by *every* point-group symmetry. In this case, the star consists of this one wave vector, and the small irrep is the same as the full irrep of the space-group. Therefore, we get
$$ \chi(pt_\vec{r}) = \chi^0(p) e^{-i\vec k\cdot \vec r} $$
for all irreps $\chi^0$ of the point group.

#### A surprising example

Consider the two-arm star $(\pi,0); (0,\pi)$ on the square lattice. The little co-group of both of these is the Vierergruppe generated by reflections $\sigma$ across the $x$ and $y$ axes. They are, however, related to one another by conjugating with a 90° rotation or a diagonal mirror, both of which exchanges $\sigma_x$ and $\sigma_y$. Therefore, the irrep $B_1$ of $P(\pi,0)$ (characters: $\chi(e) = \chi(\sigma_x) = 1$, $\chi(\sigma_y) = \chi(I) = -1$) doesn't correspond to $B_1$ at $(0,\pi)$ (as one might naively expect), but $B_2$, which flips the characters of $\sigma_{x,y}$.
Therefore, we get
$$ \begin{aligned}
\chi(t_\vec{r}) &= (-1)^{x} + (-1)^{y} \\
\chi(\sigma_xt_\vec{r}) &= (-1)^x - (-1)^y \\
\chi(\sigma_yt_\vec{r}) &= -(-1)^x + (-1)^y \\
\chi(It_\vec{r}) &= -(-1)^x - (-1)^y \\
\chi(s) &= 0\ \textrm{otherwise}.
\end{aligned} $$

## Other symmetries

In addition to space-group symmetries, many systems have discrete on-site symmetries one would like to impose to improve the quality of wave functions.

A case in point is the $\mathbb{Z}_2$ parity symmetry $\hat \pi = \prod_i \hat\sigma_i^x$ of spin-1/2 systems.
Usually, one does not try to enforce the full SU(2) spin rotation symmetry of a Heisenberg model [^vieijra]. Instead, we focus its U(1) subgroup generated by $S^z$: constraining the total $S^z$ of the system is straightforward in the usual computational basis.
However, one can show that wave functions with SU(2) spin quantum number $s$ and $S^z = 0$ are also eigenstates of parity with eigenvalue $(-1)^s$. Imposing this eigenvalue allows one to distinguish the singlet and triplet sectors of antiferromagnets, and hence extract the spin gap.

[^vieijra]: However, neural quantum states with full SU(2) symmetry have recently been [introduced](https://arxiv.org/abs/2104.14869).

Parity symmetry fits neatly in the framework we use for the space group, for it too maps each $\sigma^z$ basis state $|\boldsymbol\sigma\rangle$ to precisely one other, $|-\boldsymbol\sigma\rangle$. This commutes with every space-group symmetry, hence the total symmetry group is a direct product of the space group and the $\mathbb{Z}_2$ group generated by $\hat\pi$. NetKet already offers functionality for generating the direct product of two groups, which could be used to construct the product table used by `DenseEquivalent`; expanding `DenseSymm` should be straightforward, too.

The character table of direct products of too groups is a "direct product" of those of the factors. That is, irreps of the space-parity group are described by an irrep of the space group (characters: $\chi_0(s)$) and a parity eigenvalue $p=\pm1$, from which
$$ \begin{aligned}
\chi(s) &= \chi_0(s); & \chi(\hat\pi s) &= p\chi_0(s).
\end{aligned} $$

## From character tables to symmetric wave functions

Characters of symmetry groups can be used directly to construct wave functions that transform according to the corresponding irrep. Namely, it follows from the orthogonality properties of irreps that
$$ \mathcal{P}_\alpha |\psi\rangle = \frac{d_\alpha}{|G|} \sum_{g\in G} \chi_\alpha(g)^* g |\psi\rangle $$
is the projection of the wave function $|\psi\rangle$ onto the part of the Hilbert space that transforms according to the irrep $\alpha$ of the symmetry group $G$, which has dimension $d_\alpha$.

This motivates creating a network architecture that is able to generate the components of the wave functions $g|\psi\rangle$ for all group elements at once. NetKet offers this functionality in all its glory through *group convolutional neural networks,* which can be constructed with arbitrary width, depth, and activation functions. All that is needed is a final layer that combines them with the prefactors $\chi(g)^*$.